--[[ The Item Use utility has a dependency on SpellCast.lua and .xml.
	Its responsibility is to record indentifying information about items
	as those items are used.
]]
--[[ Multiple AddOns may include this utility.
	In order to prevent overwritten info, or just wasted cycles,
	the following definitions are only made once.
]]
if ( not ItemUse ) then

--[[ Listening-in on the activities of the player permits this utility
	to anticipate the name (or some identifying attribute) of an item before it is used.
	The following four functions should be all those that are called during item use.
]]
hooksecurefunc( "UseAction",
	function( slot )
		SpellCast_Tooltip:SetAction(slot)
		ItemUse:On()
	end
)

hooksecurefunc( "UseContainerItem",
	function( bag, slot )
		SpellCast_Tooltip:SetBagItem( bag, slot )
		ItemUse:On()
	end
)

hooksecurefunc( "UseInventoryItem",
	function( slot )
		SpellCast_Tooltip:SetInventoryItem( 'player', slot )
		ItemUse:On()
	end
)

--[[ PickupInventoryItem has to be treated differently,
	since it is replacing an item with the cursor's unknown contents.
	The ItemUse.name variable only becomes a number here, which makes it
	a good signal for the remainder of the event handling to recognize.
]]
hooksecurefunc( "PickupInventoryItem",
	function( slot )
		if ( not ItemUse.name ) then
			ItemUse:On()
			ItemUse.name	= slot
		end
	end
)

--[[ The hooked functions above are responsible for readying information
	in anticipation of an item cooldown.  The following code <reacts>
	to those cooldowns.
	
	Note that this utility does not trigger anything.
	Its purpose is to pick up an item activation that SpellCast discards,
	obtain as much info about that item as possible,
	and then renew the discarded information as a SpellCast_inProgress.
	
	AddOns that make use of this utility will be as interested
	in the UNIT_INVENTORY_CHANGED event as they are in SPELL_UPDATE_COOLDOWN.
]]
local utility	= CreateFrame( "Frame", "ItemUse" )

utility.On = function(self)
	local item	= SpellCast_TooltipTextLeft1:GetText()
	self.name 	= item
	self:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED" )
	self:RegisterEvent( "UNIT_SPELLCAST_FAILED" )
	assert( "ItemUse.On", item )
end

utility.Off = function(self)
	self.name 	= nil
	self.bag	= nil
	self:UnregisterEvent( "UNIT_SPELLCAST_INTERRUPTED" )
	self:UnregisterEvent( "UNIT_SPELLCAST_FAILED" )
end

local events	= {}
utility.events	= events
utility:RegisterEvent( "UNIT_INVENTORY_CHANGED" )
utility:RegisterEvent( "BAG_UPDATE" )
utility:RegisterEvent( "SPELL_UPDATE_COOLDOWN" )

utility:SetScript( "OnEvent",
	function(self, event, container)
		if ( self.name ) then
			assert( "ItemUse.OnEvent", event, container )
			self.events[ event ]( self, container )
		end
	end
)

--[[ BAG_UPDATEs seem to always precede UNIT_INVENTORY_CHANGED
	when bag cooldowns occur.  This is a fine time to get bag-specific
	information about the item used.
]]
events.BAG_UPDATE = function(self, bag)
	self.bag	= bag
end

events.SPELL_UPDATE_COOLDOWN	= function(self, unit)
	--[[ As was mentioned above, PickupInventoryItem,
		which is actually 'putting down' an item,
		only makes name info available after the placement completes.
		We fetch that info now.
	]]
	local itemName	= self.name
	if ( type( itemName ) == "number" ) then
		SpellCast_Tooltip:SetInventoryItem( 'player', itemName )
		itemName	= SpellCast_TooltipTextLeft1:GetText()
	end
	
	local item = {	name	= itemName	}
	local USED_ITEM_PATTERN = "(.*%[" .. itemName .. "%].*)"
	
	--[[ It's not tidy to declare a function within a function,
		but this felt like the easiest way to create a data-mining method
		that was the same for two possibilities, except for the subprocedures used.
	]]
	local function FindItem(container)
		local GetItemLink, bag
		local function InSlot(slot)
			for itemLink in string.gmatch(
				GetItemLink( container, slot ) or "", USED_ITEM_PATTERN
			) do
				assert( "ItemUse", itemLink )
				
				item.bag				= bag
				item.slot				= slot
				item.link				= itemLink
				
				SpellCast_ignored		= nil
				SpellCast_inProgress	= item
				SpellCast_last			= item
				
				self:Off()
				return true
			end
		end
			
		if ( container ~= 'player' ) then
			GetItemLink, bag	= GetContainerItemLink, container
			
			for slot = 1, GetContainerNumSlots( container ) do
				if InSlot( slot ) then	return true	end
			end
			
		else
			GetItemLink = GetInventoryItemLink
			
			--[[ This sequence has been selected to trawl though
				most-used trinket slots in order.  If the desired info
				is encountered early in the sequence, time is saved.
				unfortunately, I could not imagine a way to do this with bags.
			]]
			for slot, slot in ipairs( { 13, 14, 17, 16, 1, 5, 6, 8, 15, 11, 12, 10, 7, 2 } ) do
				if InSlot( slot ) then	return true	end
			end
		end
	end
	
	if ( not FindItem( 'player' ) ) and ( self.bag ) then
		FindItem( self.bag )
	end
end
events.UNIT_INVENTORY_CHANGED	= function(self, unit)
	if ( unit ~= 'player' ) then	return	end
	events.SPELL_UPDATE_COOLDOWN( self, unit )
end

events.UNIT_SPELLCAST_FAILED	= function(self, unit)
	if ( unit ~= 'player' ) then	return	end
	self:Off()
end
events.UNIT_SPELLCAST_INTERRUPTED	= events.UNIT_SPELLCAST_FAILED

end	-- ITEM-USE UTILITY DEFINITION
